//-----------------------------------------------------------------------------+
// version 3.3                                                                 |
//                                                                             |
// YANC.cc                                                                     |
//                                                                             |
// C++ code                                                                    |
//                                                                             |
// written by Walter Dehnen, 2000/2001                                         |
// e-mail:   dehnen@mpia.de                                                    |
// address:  Max-Planck Institut fr Astronomie, Knigstuhl 69117 Heidelberg   |
//           Germany                                                           |
//                                                                             |
//-----------------------------------------------------------------------------+
//                                                                             |
//                     YANC -- Yet Another N-body Code                         |
//                                                                             |
//-----------------------------------------------------------------------------+

#include <fstream>
#include <strstream>
#include <string>
#include <yanc.h>
#include <util/ionl.h>
#include <util/ionl.h>
using util::open;
using namespace std;
using namespace nbdy;

int main(int argc, char* argv[])
{
  if(argc < 5) {
    cout<<
      "\n usage: \"YANC FILE BASE dT dT_O [S] [F]\"\n"
      "\n Reads  parameters and  data from  FILE in yanc format  (see below) and"
      "\n integrates for time dT, making outputs every dT_O time units."
      "\n Outputs  are made in  yanc format to file  BASE.XXXX,  where XXXX is a"
      "\n (zero padded)  integer label  starting with F  (default 0).  If S > 0,"
      "\n (default=0) scratch outputs to file BASE.yanc are made each blockstep."
      "\n\n";
    yanc::describe_yanc_format(cout);
    exit(1);
  }

  ofstream scratch;
  string   base,ofile,sfile;
  char     extn[10] = ".";
  float
    dTot  = atof(argv[3]),
    dTou  = atof(argv[4]);
  uint
    Nscr  = (argc>=6)? atoi(argv[5]) : 0,
    out1  = (argc>=7)? atoi(argv[6]) : 0;
  bool  scratchout  = ( Nscr>0 );

  base  = argv[2];
  sfile = base + ".yanc";
  if(scratchout && !open(scratch,sfile)) {
    scratchout = false;
    cerr<<" ==> won't make scratch outputs\n";
  }
  scratch.close();

  yanc YANC(argv[1]);
  if(!YANC.okay()) exit(1);
  register uint o = out1;
  register float Tout = YANC.initial_time() + 0.9999*dTou;
  const    float Tend = YANC.initial_time() + dTot;
  YANC.describe(cout);
  YANC.short_head(cout);
  YANC.short_stats(cout);
  {
    ostrstream ostr(extn+1,9);
    ostr.width(4);
    ostr.fill('0');
    ostr<<o++;
    ofile = base + extn;
    YANC.write_yanc_ascii(ofile.c_str(),"YANC");
  }
  while(YANC.time() < Tend) {
    YANC.full_step();
    YANC.short_stats(cout);
    if(YANC.time() >= Tout) {
      ostrstream ostr(extn+1,9);
      ostr.width(4);
      ostr.fill('0');
      ostr<<o++;
      ofile = base + extn;
      YANC.write_yanc_ascii(ofile.c_str(),"YANC");
      Tout += dTou;
    }
    if(scratchout) YANC.write_yanc_binary(sfile.c_str(),"YANC");
  };
}
//---------------------end-of-YANC.cc------that's-it-!--------------------------
